"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[1117],{71117(e){e.exports=JSON.parse('{"grafana-sql":{"components":{"confirm-modal":{"builder-mode":"Re\u017Eim n\xE1stroje pro tvorbu nezobrazuje zm\u011Bny proveden\xE9 v\xA0k\xF3du. N\xE1stroj pro tvorbu dotaz\u016F zobraz\xED posledn\xED zm\u011Bny proveden\xE9 v\xA0re\u017Eimu n\xE1stroje pro tvorbu.","cancel":"Zru\u0161it","clipboard":"Chcete zkop\xEDrovat k\xF3d do schr\xE1nky?","copy-code-and-switch":"Kop\xEDrovat k\xF3d a\xA0p\u0159epnout","discard-code-and-switch":"Zahodit k\xF3d a\xA0p\u0159epnout","warning":"Varov\xE1n\xED"},"connection-limits":{"auto-max-idle":"Auto. max. ne\u010Dinn\xFD","content-auto-max-idle":"","content-max-idle":"","content-max-lifetime":"Maxim\xE1ln\xED doba v\xA0sekund\xE1ch, po kterou m\u016F\u017Ee b\xFDt p\u0159ipojen\xED znovu pou\u017Eito. Pokud je nastaveno na 0, p\u0159ipojen\xED jsou znovu pou\u017E\xEDv\xE1na opakovan\u011B.","content-max-open":"","max-idle":"Max. ne\u010Dinn\xFD","max-lifetime":"Max. \u017Eivotnost","max-open":"Max. otev\u0159en\xFD","title-connection-limits":"Omezen\xED spojen\xED"},"dataset-selector":{"aria-label-dataset-selector":"Voli\u010D datov\xE9 sady"},"error-boundary":{"fall-back":{"error":"Chyba"}},"get-custom-operators":{"custom-operators":{"label":{"macros":"Makra"}}},"make-render-column":{"render-column":{"aria-label-group-by":"Se\u0159adit podle","title-remove-group-by-column":"Odebrat skupinu podle sloupce"}},"order-by-row":{"aria-label-order-by":"\u0158adit podle","label-limit":"Limit","label-offset":"Odchylka","label-order-by":"\u0158adit podle"},"preview":{"label-element":{"preview":"N\xE1hled","tooltip-copy-to-clipboard":"Kop\xEDrovat do schr\xE1nky"}},"query-header":{"content-invalid-query":"","editor-modes":{"label-builder":"N\xE1stroj pro tvorbu","label-code":"K\xF3d"},"label-dataset":"Soubor dat","label-filter":"Filtrovat","label-format":"Form\xE1t","label-group":"Skupina","label-order":"Objedn\xE1vka","label-preview":"N\xE1hled","label-table":"Tabulka","placeholder-select-format":"Zvolit form\xE1t","run-query":"Spustit dotaz"},"query-meta":{"variables":{"textField":"","valueField":""}},"query-toolbox":{"content-hit-ctrlcmdreturn-to-run-query":"Stiskn\u011Bte CTRL/CMD\xA0+ Return pro spu\u0161t\u011Bn\xED dotazu","tooltip-collapse":"Sbalit editor","tooltip-expand":"Rozbalit editor","tooltip-format-query":"Form\xE1tovat dotaz"},"query-validator":{"query-will-process":"","validating-query":"Ov\u011B\u0159ov\xE1n\xED dotazu\u2026"},"raw-editor":{"render-placeholder":{"editing-in-expanded-code-editor":"\xDApravy v\xA0roz\u0161\xED\u0159en\xE9m editoru k\xF3du"},"title-query-num":"Dotaz {{queryNum}}"},"select-column":{"label-column":"Sloupec"},"select-custom-function-parameters":{"aria-label-parameter":"Parametr {{index}} pro sloupec {{columnIndex}}","render-parameters":{"params":{"title-remove-parameter":"Odebrat parametr"}},"title-add-parameter":"P\u0159idat parametr"},"select-row":{"aggregate-options":{"options":{"label":{"aggregations":"Agregace","macros":"Makra"}}},"label":{"time":"\u010Das","value":"hodnota"},"label-alias":"P\u0159ezd\xEDvka","label-data-operations":"Datov\xE9 operace","title-add-column":"P\u0159idat sloupec","title-remove-column":"Odebrat sloupec"},"settings":{"aria-label-conjunction":"Konjunkce","aria-label-field":"Pole","aria-label-operator":"Oper\xE1tor","title-button-filter":"{{ buttonLabel }} filtr"},"sql-query-editor-lazy":{"text-loading-editor":"Na\u010D\xEDt\xE1n\xED editoru"},"table-selector":{"aria-label-table-selector":"Voli\u010D tabulky","placeholder-loading":"Na\u010D\xEDt\xE1n\xED tabulek","placeholder-select-table":"Vyberte tabulku"},"tlssecrets-config":{"content-tlsssl-client-certificate":"Pro ov\u011B\u0159en\xED pomoc\xED klientsk\xE9ho certifik\xE1tu TLS/SSL zadejte klientsk\xFD certifik\xE1t zde.","content-tlsssl-client-key":"Pro ov\u011B\u0159en\xED pomoc\xED klientsk\xE9ho certifik\xE1tu TLS/SSL zadejte kl\xED\u010D zde.","content-tlsssl-root-certificate":"Pokud vybran\xFD re\u017Eim TLS/SSL vy\u017Eaduje ko\u0159enov\xFD certifik\xE1t serveru, zadejte ho zde","tlsssl-client-certificate":"Certifik\xE1t klienta TLS/SSL","tlsssl-client-key":"Kl\xED\u010D klienta TLS/SSL","tlsssl-root-certificate":"Ko\u0159enov\xFD certifik\xE1t TLS/SSL"},"visual-editor":{"label-filter-by-column-value":"Filtrovat podle hodnoty sloupce","label-group-by-column":"Se\u0159adit podle sloupce"},"widgets":{"aria-label-macros-value-selector":"Voli\u010D hodnoty maker"}},"utils":{"get-columns-width-indices":{"label-selected-columns":"Vybran\xE9 sloupce"}}}}')}}]);

//# sourceMappingURL=1117.fdb188c5c7c67b2089fd.js.map