"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[8463],{21369(m,t,s){s.r(t),s.d(t,{plugin:()=>d});var o=s(16207),a=s(74848),e=s(22803),i=s(92745),r=s(63142);const c=[{value:0,label:"Documentation",href:"https://grafana.com/docs/grafana/latest"},{value:1,label:"Tutorials",href:"https://grafana.com/tutorials"},{value:2,label:"Community",href:"https://community.grafana.com"},{value:3,label:"Public Slack",href:"http://slack.grafana.com"}],p=()=>{const n=(0,r.of)(g);return(0,a.jsxs)("div",{className:n.container,children:[(0,a.jsx)("h1",{className:n.title,children:(0,a.jsx)(i.x6,{i18nKey:"welcome.welcome-banner.welcome-to-grafana",children:"Welcome to Grafana"})}),(0,a.jsxs)("div",{className:n.help,children:[(0,a.jsx)("h2",{className:n.helpText,children:(0,a.jsx)(i.x6,{i18nKey:"welcome.welcome-banner.need-help",children:"Need help?"})}),(0,a.jsx)("div",{className:n.helpLinks,children:c.map((l,f)=>(0,a.jsx)("a",{className:n.helpLink,href:`${l.href}?utm_source=grafana_gettingstarted`,children:l.label},`${l.label}-${f}`))})]})]})},g=n=>({container:(0,e.css)({display:"flex",backgroundSize:"cover",height:"100%",alignItems:"center",justifyContent:"space-between",padding:n.spacing(0,3),[n.breakpoints.down("lg")]:{backgroundPosition:"0px",flexDirection:"column",alignItems:"flex-start",justifyContent:"center"},[n.breakpoints.down("sm")]:{padding:n.spacing(0,1)}}),title:(0,e.css)({marginBottom:0,[n.breakpoints.down("lg")]:{marginBottom:n.spacing(1)},[n.breakpoints.down("md")]:{fontSize:n.typography.h2.fontSize},[n.breakpoints.down("sm")]:{fontSize:n.typography.h3.fontSize}}),help:(0,e.css)({display:"flex",alignItems:"baseline"}),helpText:(0,e.css)({...n.typography.h3,marginRight:n.spacing(2),marginBottom:0,[n.breakpoints.down("md")]:{fontSize:n.typography.h4.fontSize},[n.breakpoints.down("sm")]:{display:"none"}}),helpLinks:(0,e.css)({display:"flex",flexWrap:"wrap"}),helpLink:(0,e.css)({marginRight:n.spacing(2),textDecoration:"underline",textWrap:"nowrap",[n.breakpoints.down("sm")]:{marginRight:n.spacing(1)}})}),d=new o.m(p).setNoPadding()}}]);

//# sourceMappingURL=welcomeBanner.103426aa0e53dbf255aa.js.map