// font-awesome icons are deprecated
// DO NOT move these to emotion globals
// because of the amount of styles here, it causes performance issues
// TODO remove this once we've communicated a proper deprecation and removal strategy
/*!
 *  Font Awesome 4.7.0 by @davegandy - http://fontawesome.io - @fontawesome
 *  License - http://fontawesome.io/license (Font: SIL OFL 1.1, CSS: MIT License)
 */

$fa-font-path: '../fonts' !default;
$fa-font-size-base: 14px !default;
$fa-line-height-base: 1 !default;
$fa-css-prefix: fa !default;
$fa-version: '4.7.0' !default;
$fa-border-color: #eee !default;
$fa-inverse: #fff !default;
$fa-li-width: calc(30em / 14) !default;

@font-face {
  font-family: 'FontAwesome';
  src: url('#{$fa-font-path}/fontawesome-webfont.eot?v=#{$fa-version}');
  src:
    url('#{$fa-font-path}/fontawesome-webfont.eot?#iefix&v=#{$fa-version}') format('embedded-opentype'),
    url('#{$fa-font-path}/fontawesome-webfont.woff2?v=#{$fa-version}') format('woff2'),
    url('#{$fa-font-path}/fontawesome-webfont.woff?v=#{$fa-version}') format('woff'),
    url('#{$fa-font-path}/fontawesome-webfont.ttf?v=#{$fa-version}') format('truetype'),
    url('#{$fa-font-path}/fontawesome-webfont.svg?v=#{$fa-version}#fontawesomeregular') format('svg');
  font-weight: normal;
  font-style: normal;
}

.#{$fa-css-prefix} {
  display: inline-block;
  font: normal normal normal #{$fa-font-size-base}/#{$fa-line-height-base} FontAwesome; // shortening font declaration
  font-size: inherit; // can't have font-size inherit on line above, so need to override
  text-rendering: auto; // optimizelegibility throws things off #1094
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.#{$fa-css-prefix}-glass:before {
  content: '\f000';
}

.#{$fa-css-prefix}-music:before {
  content: '\f001';
}

.#{$fa-css-prefix}-search:before {
  content: '\f002';
}

.#{$fa-css-prefix}-envelope-o:before {
  content: '\f003';
}

.#{$fa-css-prefix}-heart:before {
  content: '\f004';
}

.#{$fa-css-prefix}-star:before {
  content: '\f005';
}

.#{$fa-css-prefix}-star-o:before {
  content: '\f006';
}

.#{$fa-css-prefix}-user:before {
  content: '\f007';
}

.#{$fa-css-prefix}-film:before {
  content: '\f008';
}

.#{$fa-css-prefix}-th-large:before {
  content: '\f009';
}

.#{$fa-css-prefix}-th:before {
  content: '\f00a';
}

.#{$fa-css-prefix}-th-list:before {
  content: '\f00b';
}

.#{$fa-css-prefix}-check:before {
  content: '\f00c';
}

.#{$fa-css-prefix}-remove:before,
.#{$fa-css-prefix}-close:before,
.#{$fa-css-prefix}-times:before {
  content: '\f00d';
}

.#{$fa-css-prefix}-search-plus:before {
  content: '\f00e';
}

.#{$fa-css-prefix}-search-minus:before {
  content: '\f010';
}

.#{$fa-css-prefix}-power-off:before {
  content: '\f011';
}

.#{$fa-css-prefix}-signal:before {
  content: '\f012';
}

.#{$fa-css-prefix}-gear:before,
.#{$fa-css-prefix}-cog:before {
  content: '\f013';
}

.#{$fa-css-prefix}-trash-o:before {
  content: '\f014';
}

.#{$fa-css-prefix}-home:before {
  content: '\f015';
}

.#{$fa-css-prefix}-file-o:before {
  content: '\f016';
}

.#{$fa-css-prefix}-clock-o:before {
  content: '\f017';
}

.#{$fa-css-prefix}-road:before {
  content: '\f018';
}

.#{$fa-css-prefix}-download:before {
  content: '\f019';
}

.#{$fa-css-prefix}-arrow-circle-o-down:before {
  content: '\f01a';
}

.#{$fa-css-prefix}-arrow-circle-o-up:before {
  content: '\f01b';
}

.#{$fa-css-prefix}-inbox:before {
  content: '\f01c';
}

.#{$fa-css-prefix}-play-circle-o:before {
  content: '\f01d';
}

.#{$fa-css-prefix}-rotate-right:before,
.#{$fa-css-prefix}-repeat:before {
  content: '\f01e';
}

.#{$fa-css-prefix}-refresh:before {
  content: '\f021';
}

.#{$fa-css-prefix}-list-alt:before {
  content: '\f022';
}

.#{$fa-css-prefix}-lock:before {
  content: '\f023';
}

.#{$fa-css-prefix}-flag:before {
  content: '\f024';
}

.#{$fa-css-prefix}-headphones:before {
  content: '\f025';
}

.#{$fa-css-prefix}-volume-off:before {
  content: '\f026';
}

.#{$fa-css-prefix}-volume-down:before {
  content: '\f027';
}

.#{$fa-css-prefix}-volume-up:before {
  content: '\f028';
}

.#{$fa-css-prefix}-qrcode:before {
  content: '\f029';
}

.#{$fa-css-prefix}-barcode:before {
  content: '\f02a';
}

.#{$fa-css-prefix}-tag:before {
  content: '\f02b';
}

.#{$fa-css-prefix}-tags:before {
  content: '\f02c';
}

.#{$fa-css-prefix}-book:before {
  content: '\f02d';
}

.#{$fa-css-prefix}-bookmark:before {
  content: '\f02e';
}

.#{$fa-css-prefix}-print:before {
  content: '\f02f';
}

.#{$fa-css-prefix}-camera:before {
  content: '\f030';
}

.#{$fa-css-prefix}-font:before {
  content: '\f031';
}

.#{$fa-css-prefix}-bold:before {
  content: '\f032';
}

.#{$fa-css-prefix}-italic:before {
  content: '\f033';
}

.#{$fa-css-prefix}-text-height:before {
  content: '\f034';
}

.#{$fa-css-prefix}-text-width:before {
  content: '\f035';
}

.#{$fa-css-prefix}-align-left:before {
  content: '\f036';
}

.#{$fa-css-prefix}-align-center:before {
  content: '\f037';
}

.#{$fa-css-prefix}-align-right:before {
  content: '\f038';
}

.#{$fa-css-prefix}-align-justify:before {
  content: '\f039';
}

.#{$fa-css-prefix}-list:before {
  content: '\f03a';
}

.#{$fa-css-prefix}-dedent:before,
.#{$fa-css-prefix}-outdent:before {
  content: '\f03b';
}

.#{$fa-css-prefix}-indent:before {
  content: '\f03c';
}

.#{$fa-css-prefix}-video-camera:before {
  content: '\f03d';
}

.#{$fa-css-prefix}-photo:before,
.#{$fa-css-prefix}-image:before,
.#{$fa-css-prefix}-picture-o:before {
  content: '\f03e';
}

.#{$fa-css-prefix}-pencil:before {
  content: '\f040';
}

.#{$fa-css-prefix}-map-marker:before {
  content: '\f041';
}

.#{$fa-css-prefix}-adjust:before {
  content: '\f042';
}

.#{$fa-css-prefix}-tint:before {
  content: '\f043';
}

.#{$fa-css-prefix}-edit:before,
.#{$fa-css-prefix}-pencil-square-o:before {
  content: '\f044';
}

.#{$fa-css-prefix}-share-square-o:before {
  content: '\f045';
}

.#{$fa-css-prefix}-check-square-o:before {
  content: '\f046';
}

.#{$fa-css-prefix}-arrows:before {
  content: '\f047';
}

.#{$fa-css-prefix}-step-backward:before {
  content: '\f048';
}

.#{$fa-css-prefix}-fast-backward:before {
  content: '\f049';
}

.#{$fa-css-prefix}-backward:before {
  content: '\f04a';
}

.#{$fa-css-prefix}-play:before {
  content: '\f04b';
}

.#{$fa-css-prefix}-pause:before {
  content: '\f04c';
}

.#{$fa-css-prefix}-stop:before {
  content: '\f04d';
}

.#{$fa-css-prefix}-forward:before {
  content: '\f04e';
}

.#{$fa-css-prefix}-fast-forward:before {
  content: '\f050';
}

.#{$fa-css-prefix}-step-forward:before {
  content: '\f051';
}

.#{$fa-css-prefix}-eject:before {
  content: '\f052';
}

.#{$fa-css-prefix}-chevron-left:before {
  content: '\f053';
}

.#{$fa-css-prefix}-chevron-right:before {
  content: '\f054';
}

.#{$fa-css-prefix}-plus-circle:before {
  content: '\f055';
}

.#{$fa-css-prefix}-minus-circle:before {
  content: '\f056';
}

.#{$fa-css-prefix}-times-circle:before {
  content: '\f057';
}

.#{$fa-css-prefix}-check-circle:before {
  content: '\f058';
}

.#{$fa-css-prefix}-question-circle:before {
  content: '\f059';
}

.#{$fa-css-prefix}-info-circle:before {
  content: '\f05a';
}

.#{$fa-css-prefix}-crosshairs:before {
  content: '\f05b';
}

.#{$fa-css-prefix}-times-circle-o:before {
  content: '\f05c';
}

.#{$fa-css-prefix}-check-circle-o:before {
  content: '\f05d';
}

.#{$fa-css-prefix}-ban:before {
  content: '\f05e';
}

.#{$fa-css-prefix}-arrow-left:before {
  content: '\f060';
}

.#{$fa-css-prefix}-arrow-right:before {
  content: '\f061';
}

.#{$fa-css-prefix}-arrow-up:before {
  content: '\f062';
}

.#{$fa-css-prefix}-arrow-down:before {
  content: '\f063';
}

.#{$fa-css-prefix}-mail-forward:before,
.#{$fa-css-prefix}-share:before {
  content: '\f064';
}

.#{$fa-css-prefix}-expand:before {
  content: '\f065';
}

.#{$fa-css-prefix}-compress:before {
  content: '\f066';
}

.#{$fa-css-prefix}-plus:before {
  content: '\f067';
}

.#{$fa-css-prefix}-minus:before {
  content: '\f068';
}

.#{$fa-css-prefix}-asterisk:before {
  content: '\f069';
}

.#{$fa-css-prefix}-exclamation-circle:before {
  content: '\f06a';
}

.#{$fa-css-prefix}-gift:before {
  content: '\f06b';
}

.#{$fa-css-prefix}-leaf:before {
  content: '\f06c';
}

.#{$fa-css-prefix}-fire:before {
  content: '\f06d';
}

.#{$fa-css-prefix}-eye:before {
  content: '\f06e';
}

.#{$fa-css-prefix}-eye-slash:before {
  content: '\f070';
}

.#{$fa-css-prefix}-warning:before,
.#{$fa-css-prefix}-exclamation-triangle:before {
  content: '\f071';
}

.#{$fa-css-prefix}-plane:before {
  content: '\f072';
}

.#{$fa-css-prefix}-calendar:before {
  content: '\f073';
}

.#{$fa-css-prefix}-random:before {
  content: '\f074';
}

.#{$fa-css-prefix}-comment:before {
  content: '\f075';
}

.#{$fa-css-prefix}-magnet:before {
  content: '\f076';
}

.#{$fa-css-prefix}-chevron-up:before {
  content: '\f077';
}

.#{$fa-css-prefix}-chevron-down:before {
  content: '\f078';
}

.#{$fa-css-prefix}-retweet:before {
  content: '\f079';
}

.#{$fa-css-prefix}-shopping-cart:before {
  content: '\f07a';
}

.#{$fa-css-prefix}-folder:before {
  content: '\f07b';
}

.#{$fa-css-prefix}-folder-open:before {
  content: '\f07c';
}

.#{$fa-css-prefix}-arrows-v:before {
  content: '\f07d';
}

.#{$fa-css-prefix}-arrows-h:before {
  content: '\f07e';
}

.#{$fa-css-prefix}-bar-chart-o:before,
.#{$fa-css-prefix}-bar-chart:before {
  content: '\f080';
}

.#{$fa-css-prefix}-twitter-square:before {
  content: '\f081';
}

.#{$fa-css-prefix}-facebook-square:before {
  content: '\f082';
}

.#{$fa-css-prefix}-camera-retro:before {
  content: '\f083';
}

.#{$fa-css-prefix}-key:before {
  content: '\f084';
}

.#{$fa-css-prefix}-gears:before,
.#{$fa-css-prefix}-cogs:before {
  content: '\f085';
}

.#{$fa-css-prefix}-comments:before {
  content: '\f086';
}

.#{$fa-css-prefix}-thumbs-o-up:before {
  content: '\f087';
}

.#{$fa-css-prefix}-thumbs-o-down:before {
  content: '\f088';
}

.#{$fa-css-prefix}-star-half:before {
  content: '\f089';
}

.#{$fa-css-prefix}-heart-o:before {
  content: '\f08a';
}

.#{$fa-css-prefix}-sign-out:before {
  content: '\f08b';
}

.#{$fa-css-prefix}-linkedin-square:before {
  content: '\f08c';
}

.#{$fa-css-prefix}-thumb-tack:before {
  content: '\f08d';
}

.#{$fa-css-prefix}-external-link:before {
  content: '\f08e';
}

.#{$fa-css-prefix}-sign-in:before {
  content: '\f090';
}

.#{$fa-css-prefix}-trophy:before {
  content: '\f091';
}

.#{$fa-css-prefix}-github-square:before {
  content: '\f092';
}

.#{$fa-css-prefix}-upload:before {
  content: '\f093';
}

.#{$fa-css-prefix}-lemon-o:before {
  content: '\f094';
}

.#{$fa-css-prefix}-phone:before {
  content: '\f095';
}

.#{$fa-css-prefix}-square-o:before {
  content: '\f096';
}

.#{$fa-css-prefix}-bookmark-o:before {
  content: '\f097';
}

.#{$fa-css-prefix}-phone-square:before {
  content: '\f098';
}

.#{$fa-css-prefix}-twitter:before {
  content: '\f099';
}

.#{$fa-css-prefix}-facebook-f:before,
.#{$fa-css-prefix}-facebook:before {
  content: '\f09a';
}

.#{$fa-css-prefix}-github:before {
  content: '\f09b';
}

.#{$fa-css-prefix}-unlock:before {
  content: '\f09c';
}

.#{$fa-css-prefix}-credit-card:before {
  content: '\f09d';
}

.#{$fa-css-prefix}-feed:before,
.#{$fa-css-prefix}-rss:before {
  content: '\f09e';
}

.#{$fa-css-prefix}-hdd-o:before {
  content: '\f0a0';
}

.#{$fa-css-prefix}-bullhorn:before {
  content: '\f0a1';
}

.#{$fa-css-prefix}-bell:before {
  content: '\f0f3';
}

.#{$fa-css-prefix}-certificate:before {
  content: '\f0a3';
}

.#{$fa-css-prefix}-hand-o-right:before {
  content: '\f0a4';
}

.#{$fa-css-prefix}-hand-o-left:before {
  content: '\f0a5';
}

.#{$fa-css-prefix}-hand-o-up:before {
  content: '\f0a6';
}

.#{$fa-css-prefix}-hand-o-down:before {
  content: '\f0a7';
}

.#{$fa-css-prefix}-arrow-circle-left:before {
  content: '\f0a8';
}

.#{$fa-css-prefix}-arrow-circle-right:before {
  content: '\f0a9';
}

.#{$fa-css-prefix}-arrow-circle-up:before {
  content: '\f0aa';
}

.#{$fa-css-prefix}-arrow-circle-down:before {
  content: '\f0ab';
}

.#{$fa-css-prefix}-globe:before {
  content: '\f0ac';
}

.#{$fa-css-prefix}-wrench:before {
  content: '\f0ad';
}

.#{$fa-css-prefix}-tasks:before {
  content: '\f0ae';
}

.#{$fa-css-prefix}-filter:before {
  content: '\f0b0';
}

.#{$fa-css-prefix}-briefcase:before {
  content: '\f0b1';
}

.#{$fa-css-prefix}-arrows-alt:before {
  content: '\f0b2';
}

.#{$fa-css-prefix}-group:before,
.#{$fa-css-prefix}-users:before {
  content: '\f0c0';
}

.#{$fa-css-prefix}-chain:before,
.#{$fa-css-prefix}-link:before {
  content: '\f0c1';
}

.#{$fa-css-prefix}-cloud:before {
  content: '\f0c2';
}

.#{$fa-css-prefix}-flask:before {
  content: '\f0c3';
}

.#{$fa-css-prefix}-cut:before,
.#{$fa-css-prefix}-scissors:before {
  content: '\f0c4';
}

.#{$fa-css-prefix}-copy:before,
.#{$fa-css-prefix}-files-o:before {
  content: '\f0c5';
}

.#{$fa-css-prefix}-paperclip:before {
  content: '\f0c6';
}

.#{$fa-css-prefix}-save:before,
.#{$fa-css-prefix}-floppy-o:before {
  content: '\f0c7';
}

.#{$fa-css-prefix}-square:before {
  content: '\f0c8';
}

.#{$fa-css-prefix}-navicon:before,
.#{$fa-css-prefix}-reorder:before,
.#{$fa-css-prefix}-bars:before {
  content: '\f0c9';
}

.#{$fa-css-prefix}-list-ul:before {
  content: '\f0ca';
}

.#{$fa-css-prefix}-list-ol:before {
  content: '\f0cb';
}

.#{$fa-css-prefix}-strikethrough:before {
  content: '\f0cc';
}

.#{$fa-css-prefix}-underline:before {
  content: '\f0cd';
}

.#{$fa-css-prefix}-table:before {
  content: '\f0ce';
}

.#{$fa-css-prefix}-magic:before {
  content: '\f0d0';
}

.#{$fa-css-prefix}-truck:before {
  content: '\f0d1';
}

.#{$fa-css-prefix}-pinterest:before {
  content: '\f0d2';
}

.#{$fa-css-prefix}-pinterest-square:before {
  content: '\f0d3';
}

.#{$fa-css-prefix}-google-plus-square:before {
  content: '\f0d4';
}

.#{$fa-css-prefix}-google-plus:before {
  content: '\f0d5';
}

.#{$fa-css-prefix}-money:before {
  content: '\f0d6';
}

.#{$fa-css-prefix}-caret-down:before {
  content: '\f0d7';
}

.#{$fa-css-prefix}-caret-up:before {
  content: '\f0d8';
}

.#{$fa-css-prefix}-caret-left:before {
  content: '\f0d9';
}

.#{$fa-css-prefix}-caret-right:before {
  content: '\f0da';
}

.#{$fa-css-prefix}-columns:before {
  content: '\f0db';
}

.#{$fa-css-prefix}-unsorted:before,
.#{$fa-css-prefix}-sort:before {
  content: '\f0dc';
}

.#{$fa-css-prefix}-sort-down:before,
.#{$fa-css-prefix}-sort-desc:before {
  content: '\f0dd';
}

.#{$fa-css-prefix}-sort-up:before,
.#{$fa-css-prefix}-sort-asc:before {
  content: '\f0de';
}

.#{$fa-css-prefix}-envelope:before {
  content: '\f0e0';
}

.#{$fa-css-prefix}-linkedin:before {
  content: '\f0e1';
}

.#{$fa-css-prefix}-rotate-left:before,
.#{$fa-css-prefix}-undo:before {
  content: '\f0e2';
}

.#{$fa-css-prefix}-legal:before,
.#{$fa-css-prefix}-gavel:before {
  content: '\f0e3';
}

.#{$fa-css-prefix}-dashboard:before,
.#{$fa-css-prefix}-tachometer:before {
  content: '\f0e4';
}

.#{$fa-css-prefix}-comment-o:before {
  content: '\f0e5';
}

.#{$fa-css-prefix}-comments-o:before {
  content: '\f0e6';
}

.#{$fa-css-prefix}-flash:before,
.#{$fa-css-prefix}-bolt:before {
  content: '\f0e7';
}

.#{$fa-css-prefix}-sitemap:before {
  content: '\f0e8';
}

.#{$fa-css-prefix}-umbrella:before {
  content: '\f0e9';
}

.#{$fa-css-prefix}-paste:before,
.#{$fa-css-prefix}-clipboard:before {
  content: '\f0ea';
}

.#{$fa-css-prefix}-lightbulb-o:before {
  content: '\f0eb';
}

.#{$fa-css-prefix}-exchange:before {
  content: '\f0ec';
}

.#{$fa-css-prefix}-cloud-download:before {
  content: '\f0ed';
}

.#{$fa-css-prefix}-cloud-upload:before {
  content: '\f0ee';
}

.#{$fa-css-prefix}-user-md:before {
  content: '\f0f0';
}

.#{$fa-css-prefix}-stethoscope:before {
  content: '\f0f1';
}

.#{$fa-css-prefix}-suitcase:before {
  content: '\f0f2';
}

.#{$fa-css-prefix}-bell-o:before {
  content: '\f0a2';
}

.#{$fa-css-prefix}-coffee:before {
  content: '\f0f4';
}

.#{$fa-css-prefix}-cutlery:before {
  content: '\f0f5';
}

.#{$fa-css-prefix}-file-text-o:before {
  content: '\f0f6';
}

.#{$fa-css-prefix}-building-o:before {
  content: '\f0f7';
}

.#{$fa-css-prefix}-hospital-o:before {
  content: '\f0f8';
}

.#{$fa-css-prefix}-ambulance:before {
  content: '\f0f9';
}

.#{$fa-css-prefix}-medkit:before {
  content: '\f0fa';
}

.#{$fa-css-prefix}-fighter-jet:before {
  content: '\f0fb';
}

.#{$fa-css-prefix}-beer:before {
  content: '\f0fc';
}

.#{$fa-css-prefix}-h-square:before {
  content: '\f0fd';
}

.#{$fa-css-prefix}-plus-square:before {
  content: '\f0fe';
}

.#{$fa-css-prefix}-angle-double-left:before {
  content: '\f100';
}

.#{$fa-css-prefix}-angle-double-right:before {
  content: '\f101';
}

.#{$fa-css-prefix}-angle-double-up:before {
  content: '\f102';
}

.#{$fa-css-prefix}-angle-double-down:before {
  content: '\f103';
}

.#{$fa-css-prefix}-angle-left:before {
  content: '\f104';
}

.#{$fa-css-prefix}-angle-right:before {
  content: '\f105';
}

.#{$fa-css-prefix}-angle-up:before {
  content: '\f106';
}

.#{$fa-css-prefix}-angle-down:before {
  content: '\f107';
}

.#{$fa-css-prefix}-desktop:before {
  content: '\f108';
}

.#{$fa-css-prefix}-laptop:before {
  content: '\f109';
}

.#{$fa-css-prefix}-tablet:before {
  content: '\f10a';
}

.#{$fa-css-prefix}-mobile-phone:before,
.#{$fa-css-prefix}-mobile:before {
  content: '\f10b';
}

.#{$fa-css-prefix}-circle-o:before {
  content: '\f10c';
}

.#{$fa-css-prefix}-quote-left:before {
  content: '\f10d';
}

.#{$fa-css-prefix}-quote-right:before {
  content: '\f10e';
}

.#{$fa-css-prefix}-spinner:before {
  content: '\f110';
}

.#{$fa-css-prefix}-circle:before {
  content: '\f111';
}

.#{$fa-css-prefix}-mail-reply:before,
.#{$fa-css-prefix}-reply:before {
  content: '\f112';
}

.#{$fa-css-prefix}-github-alt:before {
  content: '\f113';
}

.#{$fa-css-prefix}-folder-o:before {
  content: '\f114';
}

.#{$fa-css-prefix}-folder-open-o:before {
  content: '\f115';
}

.#{$fa-css-prefix}-smile-o:before {
  content: '\f118';
}

.#{$fa-css-prefix}-frown-o:before {
  content: '\f119';
}

.#{$fa-css-prefix}-meh-o:before {
  content: '\f11a';
}

.#{$fa-css-prefix}-gamepad:before {
  content: '\f11b';
}

.#{$fa-css-prefix}-keyboard-o:before {
  content: '\f11c';
}

.#{$fa-css-prefix}-flag-o:before {
  content: '\f11d';
}

.#{$fa-css-prefix}-flag-checkered:before {
  content: '\f11e';
}

.#{$fa-css-prefix}-terminal:before {
  content: '\f120';
}

.#{$fa-css-prefix}-code:before {
  content: '\f121';
}

.#{$fa-css-prefix}-mail-reply-all:before,
.#{$fa-css-prefix}-reply-all:before {
  content: '\f122';
}

.#{$fa-css-prefix}-star-half-empty:before,
.#{$fa-css-prefix}-star-half-full:before,
.#{$fa-css-prefix}-star-half-o:before {
  content: '\f123';
}

.#{$fa-css-prefix}-location-arrow:before {
  content: '\f124';
}

.#{$fa-css-prefix}-crop:before {
  content: '\f125';
}

.#{$fa-css-prefix}-code-fork:before {
  content: '\f126';
}

.#{$fa-css-prefix}-unlink:before,
.#{$fa-css-prefix}-chain-broken:before {
  content: '\f127';
}

.#{$fa-css-prefix}-question:before {
  content: '\f128';
}

.#{$fa-css-prefix}-info:before {
  content: '\f129';
}

.#{$fa-css-prefix}-exclamation:before {
  content: '\f12a';
}

.#{$fa-css-prefix}-superscript:before {
  content: '\f12b';
}

.#{$fa-css-prefix}-subscript:before {
  content: '\f12c';
}

.#{$fa-css-prefix}-eraser:before {
  content: '\f12d';
}

.#{$fa-css-prefix}-puzzle-piece:before {
  content: '\f12e';
}

.#{$fa-css-prefix}-microphone:before {
  content: '\f130';
}

.#{$fa-css-prefix}-microphone-slash:before {
  content: '\f131';
}

.#{$fa-css-prefix}-shield:before {
  content: '\f132';
}

.#{$fa-css-prefix}-calendar-o:before {
  content: '\f133';
}

.#{$fa-css-prefix}-fire-extinguisher:before {
  content: '\f134';
}

.#{$fa-css-prefix}-rocket:before {
  content: '\f135';
}

.#{$fa-css-prefix}-maxcdn:before {
  content: '\f136';
}

.#{$fa-css-prefix}-chevron-circle-left:before {
  content: '\f137';
}

.#{$fa-css-prefix}-chevron-circle-right:before {
  content: '\f138';
}

.#{$fa-css-prefix}-chevron-circle-up:before {
  content: '\f139';
}

.#{$fa-css-prefix}-chevron-circle-down:before {
  content: '\f13a';
}

.#{$fa-css-prefix}-html5:before {
  content: '\f13b';
}

.#{$fa-css-prefix}-css3:before {
  content: '\f13c';
}

.#{$fa-css-prefix}-anchor:before {
  content: '\f13d';
}

.#{$fa-css-prefix}-unlock-alt:before {
  content: '\f13e';
}

.#{$fa-css-prefix}-bullseye:before {
  content: '\f140';
}

.#{$fa-css-prefix}-ellipsis-h:before {
  content: '\f141';
}

.#{$fa-css-prefix}-ellipsis-v:before {
  content: '\f142';
}

.#{$fa-css-prefix}-rss-square:before {
  content: '\f143';
}

.#{$fa-css-prefix}-play-circle:before {
  content: '\f144';
}

.#{$fa-css-prefix}-ticket:before {
  content: '\f145';
}

.#{$fa-css-prefix}-minus-square:before {
  content: '\f146';
}

.#{$fa-css-prefix}-minus-square-o:before {
  content: '\f147';
}

.#{$fa-css-prefix}-level-up:before {
  content: '\f148';
}

.#{$fa-css-prefix}-level-down:before {
  content: '\f149';
}

.#{$fa-css-prefix}-check-square:before {
  content: '\f14a';
}

.#{$fa-css-prefix}-pencil-square:before {
  content: '\f14b';
}

.#{$fa-css-prefix}-external-link-square:before {
  content: '\f14c';
}

.#{$fa-css-prefix}-share-square:before {
  content: '\f14d';
}

.#{$fa-css-prefix}-compass:before {
  content: '\f14e';
}

.#{$fa-css-prefix}-toggle-down:before,
.#{$fa-css-prefix}-caret-square-o-down:before {
  content: '\f150';
}

.#{$fa-css-prefix}-toggle-up:before,
.#{$fa-css-prefix}-caret-square-o-up:before {
  content: '\f151';
}

.#{$fa-css-prefix}-toggle-right:before,
.#{$fa-css-prefix}-caret-square-o-right:before {
  content: '\f152';
}

.#{$fa-css-prefix}-euro:before,
.#{$fa-css-prefix}-eur:before {
  content: '\f153';
}

.#{$fa-css-prefix}-gbp:before {
  content: '\f154';
}

.#{$fa-css-prefix}-dollar:before,
.#{$fa-css-prefix}-usd:before {
  content: '\f155';
}

.#{$fa-css-prefix}-rupee:before,
.#{$fa-css-prefix}-inr:before {
  content: '\f156';
}

.#{$fa-css-prefix}-cny:before,
.#{$fa-css-prefix}-rmb:before,
.#{$fa-css-prefix}-yen:before,
.#{$fa-css-prefix}-jpy:before {
  content: '\f157';
}

.#{$fa-css-prefix}-ruble:before,
.#{$fa-css-prefix}-rouble:before,
.#{$fa-css-prefix}-rub:before {
  content: '\f158';
}

.#{$fa-css-prefix}-won:before,
.#{$fa-css-prefix}-krw:before {
  content: '\f159';
}

.#{$fa-css-prefix}-bitcoin:before,
.#{$fa-css-prefix}-btc:before {
  content: '\f15a';
}

.#{$fa-css-prefix}-file:before {
  content: '\f15b';
}

.#{$fa-css-prefix}-file-text:before {
  content: '\f15c';
}

.#{$fa-css-prefix}-sort-alpha-asc:before {
  content: '\f15d';
}

.#{$fa-css-prefix}-sort-alpha-desc:before {
  content: '\f15e';
}

.#{$fa-css-prefix}-sort-amount-asc:before {
  content: '\f160';
}

.#{$fa-css-prefix}-sort-amount-desc:before {
  content: '\f161';
}

.#{$fa-css-prefix}-sort-numeric-asc:before {
  content: '\f162';
}

.#{$fa-css-prefix}-sort-numeric-desc:before {
  content: '\f163';
}

.#{$fa-css-prefix}-thumbs-up:before {
  content: '\f164';
}

.#{$fa-css-prefix}-thumbs-down:before {
  content: '\f165';
}

.#{$fa-css-prefix}-youtube-square:before {
  content: '\f166';
}

.#{$fa-css-prefix}-youtube:before {
  content: '\f167';
}

.#{$fa-css-prefix}-xing:before {
  content: '\f168';
}

.#{$fa-css-prefix}-xing-square:before {
  content: '\f169';
}

.#{$fa-css-prefix}-youtube-play:before {
  content: '\f16a';
}

.#{$fa-css-prefix}-dropbox:before {
  content: '\f16b';
}

.#{$fa-css-prefix}-stack-overflow:before {
  content: '\f16c';
}

.#{$fa-css-prefix}-instagram:before {
  content: '\f16d';
}

.#{$fa-css-prefix}-flickr:before {
  content: '\f16e';
}

.#{$fa-css-prefix}-adn:before {
  content: '\f170';
}

.#{$fa-css-prefix}-bitbucket:before {
  content: '\f171';
}

.#{$fa-css-prefix}-bitbucket-square:before {
  content: '\f172';
}

.#{$fa-css-prefix}-tumblr:before {
  content: '\f173';
}

.#{$fa-css-prefix}-tumblr-square:before {
  content: '\f174';
}

.#{$fa-css-prefix}-long-arrow-down:before {
  content: '\f175';
}

.#{$fa-css-prefix}-long-arrow-up:before {
  content: '\f176';
}

.#{$fa-css-prefix}-long-arrow-left:before {
  content: '\f177';
}

.#{$fa-css-prefix}-long-arrow-right:before {
  content: '\f178';
}

.#{$fa-css-prefix}-apple:before {
  content: '\f179';
}

.#{$fa-css-prefix}-windows:before {
  content: '\f17a';
}

.#{$fa-css-prefix}-android:before {
  content: '\f17b';
}

.#{$fa-css-prefix}-linux:before {
  content: '\f17c';
}

.#{$fa-css-prefix}-dribbble:before {
  content: '\f17d';
}

.#{$fa-css-prefix}-skype:before {
  content: '\f17e';
}

.#{$fa-css-prefix}-foursquare:before {
  content: '\f180';
}

.#{$fa-css-prefix}-trello:before {
  content: '\f181';
}

.#{$fa-css-prefix}-female:before {
  content: '\f182';
}

.#{$fa-css-prefix}-male:before {
  content: '\f183';
}

.#{$fa-css-prefix}-gittip:before,
.#{$fa-css-prefix}-gratipay:before {
  content: '\f184';
}

.#{$fa-css-prefix}-sun-o:before {
  content: '\f185';
}

.#{$fa-css-prefix}-moon-o:before {
  content: '\f186';
}

.#{$fa-css-prefix}-archive:before {
  content: '\f187';
}

.#{$fa-css-prefix}-bug:before {
  content: '\f188';
}

.#{$fa-css-prefix}-vk:before {
  content: '\f189';
}

.#{$fa-css-prefix}-weibo:before {
  content: '\f18a';
}

.#{$fa-css-prefix}-renren:before {
  content: '\f18b';
}

.#{$fa-css-prefix}-pagelines:before {
  content: '\f18c';
}

.#{$fa-css-prefix}-stack-exchange:before {
  content: '\f18d';
}

.#{$fa-css-prefix}-arrow-circle-o-right:before {
  content: '\f18e';
}

.#{$fa-css-prefix}-arrow-circle-o-left:before {
  content: '\f190';
}

.#{$fa-css-prefix}-toggle-left:before,
.#{$fa-css-prefix}-caret-square-o-left:before {
  content: '\f191';
}

.#{$fa-css-prefix}-dot-circle-o:before {
  content: '\f192';
}

.#{$fa-css-prefix}-wheelchair:before {
  content: '\f193';
}

.#{$fa-css-prefix}-vimeo-square:before {
  content: '\f194';
}

.#{$fa-css-prefix}-turkish-lira:before,
.#{$fa-css-prefix}-try:before {
  content: '\f195';
}

.#{$fa-css-prefix}-plus-square-o:before {
  content: '\f196';
}

.#{$fa-css-prefix}-space-shuttle:before {
  content: '\f197';
}

.#{$fa-css-prefix}-slack:before {
  content: '\f198';
}

.#{$fa-css-prefix}-envelope-square:before {
  content: '\f199';
}

.#{$fa-css-prefix}-wordpress:before {
  content: '\f19a';
}

.#{$fa-css-prefix}-openid:before {
  content: '\f19b';
}

.#{$fa-css-prefix}-institution:before,
.#{$fa-css-prefix}-bank:before,
.#{$fa-css-prefix}-university:before {
  content: '\f19c';
}

.#{$fa-css-prefix}-mortar-board:before,
.#{$fa-css-prefix}-graduation-cap:before {
  content: '\f19d';
}

.#{$fa-css-prefix}-yahoo:before {
  content: '\f19e';
}

.#{$fa-css-prefix}-google:before {
  content: '\f1a0';
}

.#{$fa-css-prefix}-reddit:before {
  content: '\f1a1';
}

.#{$fa-css-prefix}-reddit-square:before {
  content: '\f1a2';
}

.#{$fa-css-prefix}-stumbleupon-circle:before {
  content: '\f1a3';
}

.#{$fa-css-prefix}-stumbleupon:before {
  content: '\f1a4';
}

.#{$fa-css-prefix}-delicious:before {
  content: '\f1a5';
}

.#{$fa-css-prefix}-digg:before {
  content: '\f1a6';
}

.#{$fa-css-prefix}-pied-piper-pp:before {
  content: '\f1a7';
}

.#{$fa-css-prefix}-pied-piper-alt:before {
  content: '\f1a8';
}

.#{$fa-css-prefix}-drupal:before {
  content: '\f1a9';
}

.#{$fa-css-prefix}-joomla:before {
  content: '\f1aa';
}

.#{$fa-css-prefix}-language:before {
  content: '\f1ab';
}

.#{$fa-css-prefix}-fax:before {
  content: '\f1ac';
}

.#{$fa-css-prefix}-building:before {
  content: '\f1ad';
}

.#{$fa-css-prefix}-child:before {
  content: '\f1ae';
}

.#{$fa-css-prefix}-paw:before {
  content: '\f1b0';
}

.#{$fa-css-prefix}-spoon:before {
  content: '\f1b1';
}

.#{$fa-css-prefix}-cube:before {
  content: '\f1b2';
}

.#{$fa-css-prefix}-cubes:before {
  content: '\f1b3';
}

.#{$fa-css-prefix}-behance:before {
  content: '\f1b4';
}

.#{$fa-css-prefix}-behance-square:before {
  content: '\f1b5';
}

.#{$fa-css-prefix}-steam:before {
  content: '\f1b6';
}

.#{$fa-css-prefix}-steam-square:before {
  content: '\f1b7';
}

.#{$fa-css-prefix}-recycle:before {
  content: '\f1b8';
}

.#{$fa-css-prefix}-automobile:before,
.#{$fa-css-prefix}-car:before {
  content: '\f1b9';
}

.#{$fa-css-prefix}-cab:before,
.#{$fa-css-prefix}-taxi:before {
  content: '\f1ba';
}

.#{$fa-css-prefix}-tree:before {
  content: '\f1bb';
}

.#{$fa-css-prefix}-spotify:before {
  content: '\f1bc';
}

.#{$fa-css-prefix}-deviantart:before {
  content: '\f1bd';
}

.#{$fa-css-prefix}-soundcloud:before {
  content: '\f1be';
}

.#{$fa-css-prefix}-database:before {
  content: '\f1c0';
}

.#{$fa-css-prefix}-file-pdf-o:before {
  content: '\f1c1';
}

.#{$fa-css-prefix}-file-word-o:before {
  content: '\f1c2';
}

.#{$fa-css-prefix}-file-excel-o:before {
  content: '\f1c3';
}

.#{$fa-css-prefix}-file-powerpoint-o:before {
  content: '\f1c4';
}

.#{$fa-css-prefix}-file-photo-o:before,
.#{$fa-css-prefix}-file-picture-o:before,
.#{$fa-css-prefix}-file-image-o:before {
  content: '\f1c5';
}

.#{$fa-css-prefix}-file-zip-o:before,
.#{$fa-css-prefix}-file-archive-o:before {
  content: '\f1c6';
}

.#{$fa-css-prefix}-file-sound-o:before,
.#{$fa-css-prefix}-file-audio-o:before {
  content: '\f1c7';
}

.#{$fa-css-prefix}-file-movie-o:before,
.#{$fa-css-prefix}-file-video-o:before {
  content: '\f1c8';
}

.#{$fa-css-prefix}-file-code-o:before {
  content: '\f1c9';
}

.#{$fa-css-prefix}-vine:before {
  content: '\f1ca';
}

.#{$fa-css-prefix}-codepen:before {
  content: '\f1cb';
}

.#{$fa-css-prefix}-jsfiddle:before {
  content: '\f1cc';
}

.#{$fa-css-prefix}-life-bouy:before,
.#{$fa-css-prefix}-life-buoy:before,
.#{$fa-css-prefix}-life-saver:before,
.#{$fa-css-prefix}-support:before,
.#{$fa-css-prefix}-life-ring:before {
  content: '\f1cd';
}

.#{$fa-css-prefix}-circle-o-notch:before {
  content: '\f1ce';
}

.#{$fa-css-prefix}-ra:before,
.#{$fa-css-prefix}-resistance:before,
.#{$fa-css-prefix}-rebel:before {
  content: '\f1d0';
}

.#{$fa-css-prefix}-ge:before,
.#{$fa-css-prefix}-empire:before {
  content: '\f1d1';
}

.#{$fa-css-prefix}-git-square:before {
  content: '\f1d2';
}

.#{$fa-css-prefix}-git:before {
  content: '\f1d3';
}

.#{$fa-css-prefix}-y-combinator-square:before,
.#{$fa-css-prefix}-yc-square:before,
.#{$fa-css-prefix}-hacker-news:before {
  content: '\f1d4';
}

.#{$fa-css-prefix}-tencent-weibo:before {
  content: '\f1d5';
}

.#{$fa-css-prefix}-qq:before {
  content: '\f1d6';
}

.#{$fa-css-prefix}-wechat:before,
.#{$fa-css-prefix}-weixin:before {
  content: '\f1d7';
}

.#{$fa-css-prefix}-send:before,
.#{$fa-css-prefix}-paper-plane:before {
  content: '\f1d8';
}

.#{$fa-css-prefix}-send-o:before,
.#{$fa-css-prefix}-paper-plane-o:before {
  content: '\f1d9';
}

.#{$fa-css-prefix}-history:before {
  content: '\f1da';
}

.#{$fa-css-prefix}-circle-thin:before {
  content: '\f1db';
}

.#{$fa-css-prefix}-header:before {
  content: '\f1dc';
}

.#{$fa-css-prefix}-paragraph:before {
  content: '\f1dd';
}

.#{$fa-css-prefix}-sliders:before {
  content: '\f1de';
}

.#{$fa-css-prefix}-share-alt:before {
  content: '\f1e0';
}

.#{$fa-css-prefix}-share-alt-square:before {
  content: '\f1e1';
}

.#{$fa-css-prefix}-bomb:before {
  content: '\f1e2';
}

.#{$fa-css-prefix}-soccer-ball-o:before,
.#{$fa-css-prefix}-futbol-o:before {
  content: '\f1e3';
}

.#{$fa-css-prefix}-tty:before {
  content: '\f1e4';
}

.#{$fa-css-prefix}-binoculars:before {
  content: '\f1e5';
}

.#{$fa-css-prefix}-plug:before {
  content: '\f1e6';
}

.#{$fa-css-prefix}-slideshare:before {
  content: '\f1e7';
}

.#{$fa-css-prefix}-twitch:before {
  content: '\f1e8';
}

.#{$fa-css-prefix}-yelp:before {
  content: '\f1e9';
}

.#{$fa-css-prefix}-newspaper-o:before {
  content: '\f1ea';
}

.#{$fa-css-prefix}-wifi:before {
  content: '\f1eb';
}

.#{$fa-css-prefix}-calculator:before {
  content: '\f1ec';
}

.#{$fa-css-prefix}-paypal:before {
  content: '\f1ed';
}

.#{$fa-css-prefix}-google-wallet:before {
  content: '\f1ee';
}

.#{$fa-css-prefix}-cc-visa:before {
  content: '\f1f0';
}

.#{$fa-css-prefix}-cc-mastercard:before {
  content: '\f1f1';
}

.#{$fa-css-prefix}-cc-discover:before {
  content: '\f1f2';
}

.#{$fa-css-prefix}-cc-amex:before {
  content: '\f1f3';
}

.#{$fa-css-prefix}-cc-paypal:before {
  content: '\f1f4';
}

.#{$fa-css-prefix}-cc-stripe:before {
  content: '\f1f5';
}

.#{$fa-css-prefix}-bell-slash:before {
  content: '\f1f6';
}

.#{$fa-css-prefix}-bell-slash-o:before {
  content: '\f1f7';
}

.#{$fa-css-prefix}-trash:before {
  content: '\f1f8';
}

.#{$fa-css-prefix}-copyright:before {
  content: '\f1f9';
}

.#{$fa-css-prefix}-at:before {
  content: '\f1fa';
}

.#{$fa-css-prefix}-eyedropper:before {
  content: '\f1fb';
}

.#{$fa-css-prefix}-paint-brush:before {
  content: '\f1fc';
}

.#{$fa-css-prefix}-birthday-cake:before {
  content: '\f1fd';
}

.#{$fa-css-prefix}-area-chart:before {
  content: '\f1fe';
}

.#{$fa-css-prefix}-pie-chart:before {
  content: '\f200';
}

.#{$fa-css-prefix}-line-chart:before {
  content: '\f201';
}

.#{$fa-css-prefix}-lastfm:before {
  content: '\f202';
}

.#{$fa-css-prefix}-lastfm-square:before {
  content: '\f203';
}

.#{$fa-css-prefix}-toggle-off:before {
  content: '\f204';
}

.#{$fa-css-prefix}-toggle-on:before {
  content: '\f205';
}

.#{$fa-css-prefix}-bicycle:before {
  content: '\f206';
}

.#{$fa-css-prefix}-bus:before {
  content: '\f207';
}

.#{$fa-css-prefix}-ioxhost:before {
  content: '\f208';
}

.#{$fa-css-prefix}-angellist:before {
  content: '\f209';
}

.#{$fa-css-prefix}-cc:before {
  content: '\f20a';
}

.#{$fa-css-prefix}-shekel:before,
.#{$fa-css-prefix}-sheqel:before,
.#{$fa-css-prefix}-ils:before {
  content: '\f20b';
}

.#{$fa-css-prefix}-meanpath:before {
  content: '\f20c';
}

.#{$fa-css-prefix}-buysellads:before {
  content: '\f20d';
}

.#{$fa-css-prefix}-connectdevelop:before {
  content: '\f20e';
}

.#{$fa-css-prefix}-dashcube:before {
  content: '\f210';
}

.#{$fa-css-prefix}-forumbee:before {
  content: '\f211';
}

.#{$fa-css-prefix}-leanpub:before {
  content: '\f212';
}

.#{$fa-css-prefix}-sellsy:before {
  content: '\f213';
}

.#{$fa-css-prefix}-shirtsinbulk:before {
  content: '\f214';
}

.#{$fa-css-prefix}-simplybuilt:before {
  content: '\f215';
}

.#{$fa-css-prefix}-skyatlas:before {
  content: '\f216';
}

.#{$fa-css-prefix}-cart-plus:before {
  content: '\f217';
}

.#{$fa-css-prefix}-cart-arrow-down:before {
  content: '\f218';
}

.#{$fa-css-prefix}-diamond:before {
  content: '\f219';
}

.#{$fa-css-prefix}-ship:before {
  content: '\f21a';
}

.#{$fa-css-prefix}-user-secret:before {
  content: '\f21b';
}

.#{$fa-css-prefix}-motorcycle:before {
  content: '\f21c';
}

.#{$fa-css-prefix}-street-view:before {
  content: '\f21d';
}

.#{$fa-css-prefix}-heartbeat:before {
  content: '\f21e';
}

.#{$fa-css-prefix}-venus:before {
  content: '\f221';
}

.#{$fa-css-prefix}-mars:before {
  content: '\f222';
}

.#{$fa-css-prefix}-mercury:before {
  content: '\f223';
}

.#{$fa-css-prefix}-intersex:before,
.#{$fa-css-prefix}-transgender:before {
  content: '\f224';
}

.#{$fa-css-prefix}-transgender-alt:before {
  content: '\f225';
}

.#{$fa-css-prefix}-venus-double:before {
  content: '\f226';
}

.#{$fa-css-prefix}-mars-double:before {
  content: '\f227';
}

.#{$fa-css-prefix}-venus-mars:before {
  content: '\f228';
}

.#{$fa-css-prefix}-mars-stroke:before {
  content: '\f229';
}

.#{$fa-css-prefix}-mars-stroke-v:before {
  content: '\f22a';
}

.#{$fa-css-prefix}-mars-stroke-h:before {
  content: '\f22b';
}

.#{$fa-css-prefix}-neuter:before {
  content: '\f22c';
}

.#{$fa-css-prefix}-genderless:before {
  content: '\f22d';
}

.#{$fa-css-prefix}-facebook-official:before {
  content: '\f230';
}

.#{$fa-css-prefix}-pinterest-p:before {
  content: '\f231';
}

.#{$fa-css-prefix}-whatsapp:before {
  content: '\f232';
}

.#{$fa-css-prefix}-server:before {
  content: '\f233';
}

.#{$fa-css-prefix}-user-plus:before {
  content: '\f234';
}

.#{$fa-css-prefix}-user-times:before {
  content: '\f235';
}

.#{$fa-css-prefix}-hotel:before,
.#{$fa-css-prefix}-bed:before {
  content: '\f236';
}

.#{$fa-css-prefix}-viacoin:before {
  content: '\f237';
}

.#{$fa-css-prefix}-train:before {
  content: '\f238';
}

.#{$fa-css-prefix}-subway:before {
  content: '\f239';
}

.#{$fa-css-prefix}-medium:before {
  content: '\f23a';
}

.#{$fa-css-prefix}-yc:before,
.#{$fa-css-prefix}-y-combinator:before {
  content: '\f23b';
}

.#{$fa-css-prefix}-optin-monster:before {
  content: '\f23c';
}

.#{$fa-css-prefix}-opencart:before {
  content: '\f23d';
}

.#{$fa-css-prefix}-expeditedssl:before {
  content: '\f23e';
}

.#{$fa-css-prefix}-battery-4:before,
.#{$fa-css-prefix}-battery:before,
.#{$fa-css-prefix}-battery-full:before {
  content: '\f240';
}

.#{$fa-css-prefix}-battery-3:before,
.#{$fa-css-prefix}-battery-three-quarters:before {
  content: '\f241';
}

.#{$fa-css-prefix}-battery-2:before,
.#{$fa-css-prefix}-battery-half:before {
  content: '\f242';
}

.#{$fa-css-prefix}-battery-1:before,
.#{$fa-css-prefix}-battery-quarter:before {
  content: '\f243';
}

.#{$fa-css-prefix}-battery-0:before,
.#{$fa-css-prefix}-battery-empty:before {
  content: '\f244';
}

.#{$fa-css-prefix}-mouse-pointer:before {
  content: '\f245';
}

.#{$fa-css-prefix}-i-cursor:before {
  content: '\f246';
}

.#{$fa-css-prefix}-object-group:before {
  content: '\f247';
}

.#{$fa-css-prefix}-object-ungroup:before {
  content: '\f248';
}

.#{$fa-css-prefix}-sticky-note:before {
  content: '\f249';
}

.#{$fa-css-prefix}-sticky-note-o:before {
  content: '\f24a';
}

.#{$fa-css-prefix}-cc-jcb:before {
  content: '\f24b';
}

.#{$fa-css-prefix}-cc-diners-club:before {
  content: '\f24c';
}

.#{$fa-css-prefix}-clone:before {
  content: '\f24d';
}

.#{$fa-css-prefix}-balance-scale:before {
  content: '\f24e';
}

.#{$fa-css-prefix}-hourglass-o:before {
  content: '\f250';
}

.#{$fa-css-prefix}-hourglass-1:before,
.#{$fa-css-prefix}-hourglass-start:before {
  content: '\f251';
}

.#{$fa-css-prefix}-hourglass-2:before,
.#{$fa-css-prefix}-hourglass-half:before {
  content: '\f252';
}

.#{$fa-css-prefix}-hourglass-3:before,
.#{$fa-css-prefix}-hourglass-end:before {
  content: '\f253';
}

.#{$fa-css-prefix}-hourglass:before {
  content: '\f254';
}

.#{$fa-css-prefix}-hand-grab-o:before,
.#{$fa-css-prefix}-hand-rock-o:before {
  content: '\f255';
}

.#{$fa-css-prefix}-hand-stop-o:before,
.#{$fa-css-prefix}-hand-paper-o:before {
  content: '\f256';
}

.#{$fa-css-prefix}-hand-scissors-o:before {
  content: '\f257';
}

.#{$fa-css-prefix}-hand-lizard-o:before {
  content: '\f258';
}

.#{$fa-css-prefix}-hand-spock-o:before {
  content: '\f259';
}

.#{$fa-css-prefix}-hand-pointer-o:before {
  content: '\f25a';
}

.#{$fa-css-prefix}-hand-peace-o:before {
  content: '\f25b';
}

.#{$fa-css-prefix}-trademark:before {
  content: '\f25c';
}

.#{$fa-css-prefix}-registered:before {
  content: '\f25d';
}

.#{$fa-css-prefix}-creative-commons:before {
  content: '\f25e';
}

.#{$fa-css-prefix}-gg:before {
  content: '\f260';
}

.#{$fa-css-prefix}-gg-circle:before {
  content: '\f261';
}

.#{$fa-css-prefix}-tripadvisor:before {
  content: '\f262';
}

.#{$fa-css-prefix}-odnoklassniki:before {
  content: '\f263';
}

.#{$fa-css-prefix}-odnoklassniki-square:before {
  content: '\f264';
}

.#{$fa-css-prefix}-get-pocket:before {
  content: '\f265';
}

.#{$fa-css-prefix}-wikipedia-w:before {
  content: '\f266';
}

.#{$fa-css-prefix}-safari:before {
  content: '\f267';
}

.#{$fa-css-prefix}-chrome:before {
  content: '\f268';
}

.#{$fa-css-prefix}-firefox:before {
  content: '\f269';
}

.#{$fa-css-prefix}-opera:before {
  content: '\f26a';
}

.#{$fa-css-prefix}-internet-explorer:before {
  content: '\f26b';
}

.#{$fa-css-prefix}-tv:before,
.#{$fa-css-prefix}-television:before {
  content: '\f26c';
}

.#{$fa-css-prefix}-contao:before {
  content: '\f26d';
}

.#{$fa-css-prefix}-500px:before {
  content: '\f26e';
}

.#{$fa-css-prefix}-amazon:before {
  content: '\f270';
}

.#{$fa-css-prefix}-calendar-plus-o:before {
  content: '\f271';
}

.#{$fa-css-prefix}-calendar-minus-o:before {
  content: '\f272';
}

.#{$fa-css-prefix}-calendar-times-o:before {
  content: '\f273';
}

.#{$fa-css-prefix}-calendar-check-o:before {
  content: '\f274';
}

.#{$fa-css-prefix}-industry:before {
  content: '\f275';
}

.#{$fa-css-prefix}-map-pin:before {
  content: '\f276';
}

.#{$fa-css-prefix}-map-signs:before {
  content: '\f277';
}

.#{$fa-css-prefix}-map-o:before {
  content: '\f278';
}

.#{$fa-css-prefix}-map:before {
  content: '\f279';
}

.#{$fa-css-prefix}-commenting:before {
  content: '\f27a';
}

.#{$fa-css-prefix}-commenting-o:before {
  content: '\f27b';
}

.#{$fa-css-prefix}-houzz:before {
  content: '\f27c';
}

.#{$fa-css-prefix}-vimeo:before {
  content: '\f27d';
}

.#{$fa-css-prefix}-black-tie:before {
  content: '\f27e';
}

.#{$fa-css-prefix}-fonticons:before {
  content: '\f280';
}

.#{$fa-css-prefix}-reddit-alien:before {
  content: '\f281';
}

.#{$fa-css-prefix}-edge:before {
  content: '\f282';
}

.#{$fa-css-prefix}-credit-card-alt:before {
  content: '\f283';
}

.#{$fa-css-prefix}-codiepie:before {
  content: '\f284';
}

.#{$fa-css-prefix}-modx:before {
  content: '\f285';
}

.#{$fa-css-prefix}-fort-awesome:before {
  content: '\f286';
}

.#{$fa-css-prefix}-usb:before {
  content: '\f287';
}

.#{$fa-css-prefix}-product-hunt:before {
  content: '\f288';
}

.#{$fa-css-prefix}-mixcloud:before {
  content: '\f289';
}

.#{$fa-css-prefix}-scribd:before {
  content: '\f28a';
}

.#{$fa-css-prefix}-pause-circle:before {
  content: '\f28b';
}

.#{$fa-css-prefix}-pause-circle-o:before {
  content: '\f28c';
}

.#{$fa-css-prefix}-stop-circle:before {
  content: '\f28d';
}

.#{$fa-css-prefix}-stop-circle-o:before {
  content: '\f28e';
}

.#{$fa-css-prefix}-shopping-bag:before {
  content: '\f290';
}

.#{$fa-css-prefix}-shopping-basket:before {
  content: '\f291';
}

.#{$fa-css-prefix}-hashtag:before {
  content: '\f292';
}

.#{$fa-css-prefix}-bluetooth:before {
  content: '\f293';
}

.#{$fa-css-prefix}-bluetooth-b:before {
  content: '\f294';
}

.#{$fa-css-prefix}-percent:before {
  content: '\f295';
}

.#{$fa-css-prefix}-gitlab:before {
  content: '\f296';
}

.#{$fa-css-prefix}-wpbeginner:before {
  content: '\f297';
}

.#{$fa-css-prefix}-wpforms:before {
  content: '\f298';
}

.#{$fa-css-prefix}-envira:before {
  content: '\f299';
}

.#{$fa-css-prefix}-universal-access:before {
  content: '\f29a';
}

.#{$fa-css-prefix}-wheelchair-alt:before {
  content: '\f29b';
}

.#{$fa-css-prefix}-question-circle-o:before {
  content: '\f29c';
}

.#{$fa-css-prefix}-blind:before {
  content: '\f29d';
}

.#{$fa-css-prefix}-audio-description:before {
  content: '\f29e';
}

.#{$fa-css-prefix}-volume-control-phone:before {
  content: '\f2a0';
}

.#{$fa-css-prefix}-braille:before {
  content: '\f2a1';
}

.#{$fa-css-prefix}-assistive-listening-systems:before {
  content: '\f2a2';
}

.#{$fa-css-prefix}-asl-interpreting:before,
.#{$fa-css-prefix}-american-sign-language-interpreting:before {
  content: '\f2a3';
}

.#{$fa-css-prefix}-deafness:before,
.#{$fa-css-prefix}-hard-of-hearing:before,
.#{$fa-css-prefix}-deaf:before {
  content: '\f2a4';
}

.#{$fa-css-prefix}-glide:before {
  content: '\f2a5';
}

.#{$fa-css-prefix}-glide-g:before {
  content: '\f2a6';
}

.#{$fa-css-prefix}-signing:before,
.#{$fa-css-prefix}-sign-language:before {
  content: '\f2a7';
}

.#{$fa-css-prefix}-low-vision:before {
  content: '\f2a8';
}

.#{$fa-css-prefix}-viadeo:before {
  content: '\f2a9';
}

.#{$fa-css-prefix}-viadeo-square:before {
  content: '\f2aa';
}

.#{$fa-css-prefix}-snapchat:before {
  content: '\f2ab';
}

.#{$fa-css-prefix}-snapchat-ghost:before {
  content: '\f2ac';
}

.#{$fa-css-prefix}-snapchat-square:before {
  content: '\f2ad';
}

.#{$fa-css-prefix}-pied-piper:before {
  content: '\f2ae';
}

.#{$fa-css-prefix}-first-order:before {
  content: '\f2b0';
}

.#{$fa-css-prefix}-yoast:before {
  content: '\f2b1';
}

.#{$fa-css-prefix}-themeisle:before {
  content: '\f2b2';
}

.#{$fa-css-prefix}-google-plus-circle:before,
.#{$fa-css-prefix}-google-plus-official:before {
  content: '\f2b3';
}

.#{$fa-css-prefix}-fa:before,
.#{$fa-css-prefix}-font-awesome:before {
  content: '\f2b4';
}

.#{$fa-css-prefix}-handshake-o:before {
  content: '\f2b5';
}

.#{$fa-css-prefix}-envelope-open:before {
  content: '\f2b6';
}

.#{$fa-css-prefix}-envelope-open-o:before {
  content: '\f2b7';
}

.#{$fa-css-prefix}-linode:before {
  content: '\f2b8';
}

.#{$fa-css-prefix}-address-book:before {
  content: '\f2b9';
}

.#{$fa-css-prefix}-address-book-o:before {
  content: '\f2ba';
}

.#{$fa-css-prefix}-vcard:before,
.#{$fa-css-prefix}-address-card:before {
  content: '\f2bb';
}

.#{$fa-css-prefix}-vcard-o:before,
.#{$fa-css-prefix}-address-card-o:before {
  content: '\f2bc';
}

.#{$fa-css-prefix}-user-circle:before {
  content: '\f2bd';
}

.#{$fa-css-prefix}-user-circle-o:before {
  content: '\f2be';
}

.#{$fa-css-prefix}-user-o:before {
  content: '\f2c0';
}

.#{$fa-css-prefix}-id-badge:before {
  content: '\f2c1';
}

.#{$fa-css-prefix}-drivers-license:before,
.#{$fa-css-prefix}-id-card:before {
  content: '\f2c2';
}

.#{$fa-css-prefix}-drivers-license-o:before,
.#{$fa-css-prefix}-id-card-o:before {
  content: '\f2c3';
}

.#{$fa-css-prefix}-quora:before {
  content: '\f2c4';
}

.#{$fa-css-prefix}-free-code-camp:before {
  content: '\f2c5';
}

.#{$fa-css-prefix}-telegram:before {
  content: '\f2c6';
}

.#{$fa-css-prefix}-thermometer-4:before,
.#{$fa-css-prefix}-thermometer:before,
.#{$fa-css-prefix}-thermometer-full:before {
  content: '\f2c7';
}

.#{$fa-css-prefix}-thermometer-3:before,
.#{$fa-css-prefix}-thermometer-three-quarters:before {
  content: '\f2c8';
}

.#{$fa-css-prefix}-thermometer-2:before,
.#{$fa-css-prefix}-thermometer-half:before {
  content: '\f2c9';
}

.#{$fa-css-prefix}-thermometer-1:before,
.#{$fa-css-prefix}-thermometer-quarter:before {
  content: '\f2ca';
}

.#{$fa-css-prefix}-thermometer-0:before,
.#{$fa-css-prefix}-thermometer-empty:before {
  content: '\f2cb';
}

.#{$fa-css-prefix}-shower:before {
  content: '\f2cc';
}

.#{$fa-css-prefix}-bathtub:before,
.#{$fa-css-prefix}-s15:before,
.#{$fa-css-prefix}-bath:before {
  content: '\f2cd';
}

.#{$fa-css-prefix}-podcast:before {
  content: '\f2ce';
}

.#{$fa-css-prefix}-window-maximize:before {
  content: '\f2d0';
}

.#{$fa-css-prefix}-window-minimize:before {
  content: '\f2d1';
}

.#{$fa-css-prefix}-window-restore:before {
  content: '\f2d2';
}

.#{$fa-css-prefix}-times-rectangle:before,
.#{$fa-css-prefix}-window-close:before {
  content: '\f2d3';
}

.#{$fa-css-prefix}-times-rectangle-o:before,
.#{$fa-css-prefix}-window-close-o:before {
  content: '\f2d4';
}

.#{$fa-css-prefix}-bandcamp:before {
  content: '\f2d5';
}

.#{$fa-css-prefix}-grav:before {
  content: '\f2d6';
}

.#{$fa-css-prefix}-etsy:before {
  content: '\f2d7';
}

.#{$fa-css-prefix}-imdb:before {
  content: '\f2d8';
}

.#{$fa-css-prefix}-ravelry:before {
  content: '\f2d9';
}

.#{$fa-css-prefix}-eercast:before {
  content: '\f2da';
}

.#{$fa-css-prefix}-microchip:before {
  content: '\f2db';
}

.#{$fa-css-prefix}-snowflake-o:before {
  content: '\f2dc';
}

.#{$fa-css-prefix}-superpowers:before {
  content: '\f2dd';
}

.#{$fa-css-prefix}-wpexplorer:before {
  content: '\f2de';
}

.#{$fa-css-prefix}-meetup:before {
  content: '\f2e0';
}
