"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[6772],{16772(e){e.exports=JSON.parse(`{"grafana-sql":{"components":{"confirm-modal":{"builder-mode":"La modalit\xE0 generatore non visualizza le modifiche apportate al codice. Il generatore di query visualizzer\xE0 le ultime modifiche apportate in modalit\xE0 generatore.","cancel":"Annulla","clipboard":"Copiare il tuo codice negli appunti?","copy-code-and-switch":"Copia codice e cambia","discard-code-and-switch":"Elimina codice e cambia","warning":"Attenzione"},"connection-limits":{"auto-max-idle":"Inattivit\xE0 massima automatica","content-auto-max-idle":"","content-max-idle":"","content-max-lifetime":"La quantit\xE0 massima di tempo in secondi in cui una connessione pu\xF2 essere riutilizzata. Se impostato su 0, le connessioni vengono riutilizzate per sempre.","content-max-open":"","max-idle":"Inattive massime","max-lifetime":"Durata massima","max-open":"Aperte massime","title-connection-limits":"Limiti di collegamento"},"dataset-selector":{"aria-label-dataset-selector":"Selettore set di dati"},"error-boundary":{"fall-back":{"error":"Errore"}},"get-custom-operators":{"custom-operators":{"label":{"macros":"Macro"}}},"make-render-column":{"render-column":{"aria-label-group-by":"Raggruppa per","title-remove-group-by-column":"Rimuovi gruppo per colonna"}},"order-by-row":{"aria-label-order-by":"Ordina per","label-limit":"Limite","label-offset":"Compensazione","label-order-by":"Ordina per"},"preview":{"label-element":{"preview":"Anteprima","tooltip-copy-to-clipboard":"Copia negli appunti"}},"query-header":{"content-invalid-query":"","editor-modes":{"label-builder":"Builder","label-code":"Codice"},"label-dataset":"Set di dati","label-filter":"Filtro","label-format":"Formato","label-group":"Gruppo","label-order":"Ordina","label-preview":"Anteprima","label-table":"Tabella","placeholder-select-format":"Seleziona il formato","run-query":"Esegui query"},"query-meta":{"variables":{"textField":"","valueField":""}},"query-toolbox":{"content-hit-ctrlcmdreturn-to-run-query":"Premi CTRL/CMD+Invio per eseguire la query","tooltip-collapse":"Comprimi editor","tooltip-expand":"Espandi editor","tooltip-format-query":"Formato query"},"query-validator":{"query-will-process":"","validating-query":"Convalida della query in corso..."},"raw-editor":{"render-placeholder":{"editing-in-expanded-code-editor":"Modifica nell'editor di codice espanso"},"title-query-num":"Query {{queryNum}}"},"select-column":{"label-column":"Colonna"},"select-custom-function-parameters":{"aria-label-parameter":"Parametro {{index}} per colonna {{columnIndex}}","render-parameters":{"params":{"title-remove-parameter":"Rimuovi parametro"}},"title-add-parameter":"Aggiungi parametro"},"select-row":{"aggregate-options":{"options":{"label":{"aggregations":"Aggregazioni","macros":"Macro"}}},"label":{"time":"tempo","value":"valore"},"label-alias":"Nome alternativo","label-data-operations":"Operazioni sui dati","title-add-column":"Aggiungi colonna","title-remove-column":"Rimuovi colonna"},"settings":{"aria-label-conjunction":"Congiunzione","aria-label-field":"Campo","aria-label-operator":"Operatore","title-button-filter":"Filtro {{ buttonLabel }}"},"sql-query-editor-lazy":{"text-loading-editor":"Caricamento editor in corso..."},"table-selector":{"aria-label-table-selector":"Selettore tabella","placeholder-loading":"Caricamento tabelle","placeholder-select-table":"Seleziona tabella"},"tlssecrets-config":{"content-tlsssl-client-certificate":"Per autenticarsi con un certificato client TLS/SSL, fornisci qui il certificato client.","content-tlsssl-client-key":"Per autenticarsi con un certificato TLS/SSL client, fornisci qui la chiave.","content-tlsssl-root-certificate":"Se la modalit\xE0 TLS/SSL selezionata richiede un certificato radice del server, forniscilo qui","tlsssl-client-certificate":"Certificato client TLS/SSL","tlsssl-client-key":"Chiave client TLS/SSL","tlsssl-root-certificate":"Certificato radice TLS/SSL"},"visual-editor":{"label-filter-by-column-value":"Filtra per valore colonna","label-group-by-column":"Raggruppa per colonna"},"widgets":{"aria-label-macros-value-selector":"Selettore valore macro"}},"utils":{"get-columns-width-indices":{"label-selected-columns":"Colonne selezionate"}}}}`)}}]);

//# sourceMappingURL=6772.5aa0b0eedf8a03bc88d0.js.map