// Code generated - EDITING IS FUTILE. DO NOT EDIT.
//
// Generated by:
//     public/app/plugins/gen.go
// Using jennies:
//     TSTypesJenny
//     PluginTsTypesJenny
//
// Run 'make gen-cue' from repository root to regenerate.

// Generated from public/app/plugins/panel/logs/panelcfg.cue file.

import * as common from '@grafana/schema';

export interface Options {
  controlsStorageKey?: string;
  dedupStrategy: common.LogsDedupStrategy;
  detailsMode?: ('inline' | 'sidebar');
  displayedFields?: Array<string>;
  enableInfiniteScrolling?: boolean;
  enableLogDetails: boolean;
  fontSize?: ('default' | 'small');
  isFilterLabelActive?: unknown;
  logLineMenuCustomItems?: unknown;
  logRowMenuIconsAfter?: unknown;
  logRowMenuIconsBefore?: unknown;
  noInteractions?: boolean;
  /**
   * TODO: figure out how to define callbacks
   */
  onClickFilterLabel?: unknown;
  onClickFilterOutLabel?: unknown;
  onClickFilterOutString?: unknown;
  onClickFilterString?: unknown;
  onClickHideField?: unknown;
  onClickShowField?: unknown;
  onLogOptionsChange?: unknown;
  onNewLogsReceived?: unknown;
  prettifyLogMessage: boolean;
  setDisplayedFields?: unknown;
  showCommonLabels: boolean;
  showControls?: boolean;
  showFieldSelector?: boolean;
  showLabels: boolean;
  showLogAttributes?: boolean;
  showLogContextToggle: boolean;
  showTime: boolean;
  sortOrder: common.LogsSortOrder;
  syntaxHighlighting?: boolean;
  timestampResolution?: ('ms' | 'ns');
  unwrappedColumns?: boolean;
  wrapLogMessage: boolean;
}

export const defaultOptions: Partial<Options> = {
  displayedFields: [],
};
