"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[8638],{83560(g,e,o){o.d(e,{X:()=>c});var a=o(96540),s=o(36490);const c=({message:_,when:d=!0})=>{const r=s.Ny.getHistory();return(0,a.useEffect)(()=>{if(!d)return;const f=r.block(_);return()=>{f()}},[d,_,r]),null}},52918(g,e,o){o.d(e,{E:()=>D,P:()=>n});var a=o(74848),s=o(22803),c=o(72635),_=o(63704),d=o(85629),r=o(63142),f=o(12326),v=o(71542),P=o(45004);const L=d.Tp.pages.PublicDashboard,E="grafana-logo",R="https://grafana.com/?src=grafananet&cnt=kiosk-dashboard";var D=(t=>(t.Public="public",t.Kiosk="kiosk",t))(D||{});const n=function({variant:t="public",paddingX:i,useMinHeight:x,linkUrl:T,text:O,logo:u,hide:k}){const l=(0,r.of)(W=>M(W,{paddingX:i,useMinHeight:x})),m=(0,r.$j)(),A=(0,P.P)(),h=t==="kiosk";if(k||!h&&A.footerHide)return null;const y=_.sQ.sanitizeUrl(T??(h?R:A.footerLink)),K=t==="kiosk"?c.Ay.t("dashboard.kiosk.footerPoweredBy","Powered by"):void 0,b=t==="kiosk"?E:void 0,p=O!==void 0?O:K,B=p!==void 0?(0,a.jsx)("span",{className:l.text,children:p}):A.footerText,U=u!==void 0?u:b??A.footerLogo,I=U===E?m.isDark?v:f:U,C=I?_.sQ.sanitizeUrl(I):"";return(0,a.jsx)("div",{className:(0,s.cx)(l.footer,i!==void 0&&l.footerPaddingX,x&&l.footerMinHeight),"data-testid":L.footer,children:(0,a.jsxs)("a",{className:l.link,href:y,target:"_blank",rel:"noreferrer noopener",children:[B," ",C?(0,a.jsx)("img",{className:l.logoImg,alt:"",src:C}):null]})})},M=(t,i)=>({footer:(0,s.css)({display:"flex",justifyContent:"end",height:"30px",backgroundColor:t.colors.background.canvas,position:"sticky",bottom:0,zIndex:t.zIndex.navbarFixed,padding:t.spacing(.5,0)}),footerPaddingX:(0,s.css)({boxSizing:"border-box",paddingLeft:t.spacing(i.paddingX??0),paddingRight:t.spacing(i.paddingX??0)}),footerMinHeight:(0,s.css)({height:"auto",minHeight:"30px",alignItems:"center"}),link:(0,s.css)({display:"flex",alignItems:"center"}),logoImg:(0,s.css)({height:"16px",marginLeft:t.spacing(.5)}),text:(0,s.css)({color:t.colors.text.secondary,fontSize:t.typography.body.fontSize})})},45004(g,e,o){o.d(e,{P:()=>E});var a=o(74848),s=o(22803),c=o(63142),_=o(12326),d=o(71542);const r="https://grafana.com/?src=grafananet&cnt=public-dashboards",f=d,v=_,P="grafana-logo",L=n=>{const M=(0,c.$j)(),t=(0,c.of)(D),{footerHide:i,footerText:x,footerLink:T,footerLogo:O,headerLogoHide:u}=n||{footerHide:!1,footerText:"Powered by",footerLogo:P,footerLink:r,headerLogoHide:!1};return{footerHide:i,footerText:(0,a.jsx)("span",{className:t.text,children:x}),footerLogo:O===P?M.isDark?f:v:O,footerLink:T,headerLogoHide:u}};let E=()=>L();function R(n){E=()=>L(n)}const D=n=>({text:(0,s.css)({color:n.colors.text.secondary,fontSize:n.typography.body.fontSize})})},12326(g,e,o){g.exports=o.p+"static/img/grafana_text_logo_dark.70a1e5ee.svg"},71542(g,e,o){g.exports=o.p+"static/img/grafana_text_logo_light.467e047c.svg"}}]);

//# sourceMappingURL=DashboardScenePage.44304831c4756e3b8d1f.js.map